import React from 'react/addons';
import TokenAutocomplete from './';
import Token from './token';
import TestUtils from 'TestUtils';

let component;

describe('TokenAutocomplete', () => {

  afterEach(done => {
    React.unmountComponentAtNode(document.body);
    document.body.innerHTML = '';
    done();
  });


  //INITIAL STATE

  describe('by default', () => {

    beforeEach(() => {
      component = TestUtils.renderComponent(TokenAutocomplete);
    });

    //props
    it('has empty options array', () => {
        expect(component.props.options).to.be.instanceof(Array);
        expect(component.props.options).to.be.empty;
    });

    it('has empty value array by default', () => {
        expect(component.props.defaultValues).to.be.instanceof(Array);
        expect(component.props.defaultValues).to.be.empty;
    });

    it('has treshold of 3', () => {
        expect(component.props.treshold).to.be.equal(3);
    });

    it('has predefined placeholder', () => {
        const placeholder = 'add new tag';
        expect(component.props.placeholder).to.be.equal(placeholder);
        expect(component.refs.input.props.placeholder).to.equal(placeholder);
    });

    it('is not processing', () => {
        expect(component.props.processing).to.be.false;
    });

    it('is not limiting tags to options', () => {
        expect(component.props.limitToOptions).to.be.false;
    });

    //state
    it('is unfocused by default', () => {
       expect(component.state.focused).to.be.false;
    });

  });

  describe('contains', () => {

      it('input with placeholder', () => {
        const component = TestUtils.renderComponent(TokenAutocomplete);
        expect(component.refs.input).to.exist;
      });

  });

  //API
  describe('provides API', () => {

    it('to set initial focus', () => {
      const component = TestUtils.renderComponent(TokenAutocomplete, {focus: true});
      React.findDOMNode(component.refs.input);
      //TODO
    });

    it('to limit tags to options', () => {
      const component = TestUtils.renderComponent(TokenAutocomplete, {
        options: ['aaa', 'ccc'],
        defaultValues: ['bbb'],
        limitToOptions: true
      });


      expect(component.state.values.size).to.equal(1);

      TestUtils.changeInputValue(component, 'ddd');
      TestUtils.hitEnter(component);

      expect(component.state.values.size).to.equal(1);

    });

    it('to add custom tags ', () => {
      const component = TestUtils.renderComponent(TokenAutocomplete, {
        options: ['aaa', 'ccc'],
        defaultValues: ['bbb']
      });


      expect(component.state.values.size).to.equal(1);

      TestUtils.changeInputValue(component, 'ddd');
      TestUtils.hitEnter(component);

      expect(component.state.values.size).to.equal(1);

    });

  });


  //UNIT

  it('stores input value in state.inputValue', () => {

    const component = TestUtils.renderComponent(TokenAutocomplete);

    expect(component.state.inputValue).to.equal('');

    TestUtils.changeInputValue(component, 'abc');

    expect(component.state.inputValue).to.equal('abc');

  });

  describe('passes to options list', () => {

    beforeEach(() => {
      component = TestUtils.renderComponent(TokenAutocomplete, {
        defaultValues: ['a', 'b']
      });
      TestUtils.changeInputValue(component, 'def');
    });

    it('inputValue as term props', () => {
      expect(component.refs.options.props.term).to.equal('def');
    });


  });


  //FUNCTIONAL

  it('displays a list when options are provided and treshold is achieved', () => {

    const component = TestUtils.renderComponent(TokenAutocomplete);
    expect(component.refs.options).not.to.exist;

    TestUtils.changeInputValue(component, 'abc');


    expect(component.refs.options).to.exist;

  });

  it('displays a token for each passed value', done => {

    const component = TestUtils.renderComponent(TokenAutocomplete, {
      defaultValues: ['a', 'b', 'c', 'd']
    });

    let tokens = TestUtils.scryRenderedComponentsWithType(component.refs.wrapper, Token);
    expect(tokens.length).to.equal(4);
    done();
  });

  it('dont show already selected options', () => {

    const component = TestUtils.renderComponent(TokenAutocomplete, {
      options: ['aaa1', 'aaa2', 'aaa3', 'aaa4'],
      defaultValues: ['aaa1', 'aaa2', 'aaa3']
    });


    TestUtils.changeInputValue(component, 'aaa');

    expect(component.refs.options.props.options).to.include('aaa4');

  });

  it('dont show options not matching currently type value', () => {

    const component = TestUtils.renderComponent(TokenAutocomplete, {
      options: ['aaa1', 'aaa2', 'aaa3', 'aaa4', 'ddd1'],
      defaultValues: ['aaa1', 'aaa2', 'aaa3']
    });

    TestUtils.changeInputValue(component, 'aaa');

    expect(component.refs.options.props.options.length).to.equal(1);
    expect(component.refs.options.props.options).to.include('aaa4');

  });

  describe('on enter', () => {

    it('adds currently selected element on enter', () => {

      const component = TestUtils.renderComponent(TokenAutocomplete, {
        options: ['aaa', 'ccc'],
        defaultValues: ['bbb']
      });

      TestUtils.changeInputValue(component, 'aaa');
      TestUtils.hitEnter(component);

      expect(component.state.values.size).to.equal(2);

      TestUtils.changeInputValue(component, 'aaa');
      expect(component.refs.options.props.options).to.be.empty;

    });

    it('clears inputValue', () => {

      const component = TestUtils.renderComponent(TokenAutocomplete, {
        options: ['aaa'],
        defaultValues: ['bbb']
      });

      TestUtils.changeInputValue(component, 'aaa');
      TestUtils.hitEnter(component);

      expect(component.state.inputValue).to.be.empty;

    });

  });

  it('on backspace when input is empty deletes the last value', () => {

    const component = TestUtils.renderComponent(TokenAutocomplete, {
      defaultValues: ['aaa1', 'aaa2', 'aaa3']
    });

    TestUtils.hitBackspace(component);
    expect(component.state.values.size).to.equal(2);
    TestUtils.hitBackspace(component);
    expect(component.state.values.size).to.equal(1);
  });

  it('handles token removal', () => {

    const component = TestUtils.renderComponent(TokenAutocomplete, {
      defaultValues: ['aaa1']
    });

    const removeBtnNode = React.findDOMNode(component.refs.inputWrapper);
    const TokenNode = removeBtnNode.querySelectorAll('div')[0];
    const TokenRemoveBtnNode = TokenNode.querySelectorAll('.token-remove-btn')[0];

    expect(component.state.values.size).to.equal(1);
    TestUtils.Simulate.click(TokenRemoveBtnNode);

    expect(component.state.values.size).to.equal(0);

  });

  it('refocuses after token removal', () => {
    //TODO
  });

  it('hides list on blur', () => {

    const component = TestUtils.renderComponent(TokenAutocomplete, {
      options: ['aaa1', 'aaa2', 'aaa3', 'aaa4', 'ddd1']
    });

    TestUtils.changeInputValue(component, 'aaa');
    TestUtils.blur(component);

    expect(component.refs.options).not.to.exist;

  });

  it('displays processing status', () => {
    const component = TestUtils.renderComponent(TokenAutocomplete, {
      processing: true
    });
    expect(component.refs.processing).to.exist;
  });
});
